function varargout = sgget(label)
%SGGET Get the active structure in StratiGraph.  
%   StructObj = SGGET returns a Matlab structure object of the active structure
%   in StratiGraph. If no structure or an edge is active, StructObj is empty.
%
%   StructObj = SGGET(Label) specifies the label of the StratiGraph window to
%   get the data from.
%
%   See also SGSET, SGOPEN.

% This file is part of the Matlab plugin for StratiGraph.
% This file is freely-available under the modified BSD license.

narginchk(0,1);
nargoutchk(0,1);

if nargin == 0
	label = '';
end

[sgc, winID, label] = sgdata(label);

% Check that MCS Toolbox Integrator is loaded into StratiGraph
import se.umu.stratigraph.core.PluginManager;
if ~PluginManager.isLoaded(winID,'MCS Toolbox Integrator')
  error('MCS:sgset:ExtensionNotLoaded',...
        'StratiGraph extension ''MCS Toolbox Integrator'' is not loaded for window ''%s''.',label)
end

import se.umu.stratigraph.Main;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataType;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequest;
import se.umu.stratigraph.core.plugin.Result;
  
try 
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer
  jrc  = MCSRequestDataContainer(MCSRequestDataType.GET_STRUCTURE);
  % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequest
  jre  = MCSRequest(winID, jrc);
  
  jres = Main.callExtension(jre); % Get Result

  if jres.isSuccess()
    jstrCont = jres.getData(); % Get StructureContainer
    if ~jstrCont.hasData()
      str = [];
    else
      str = jstruct2mlstruct(jstrCont);
    end
  else
    JE = jres.getErrorCause();
    if isa(JE,'se.umu.stratigraph.plugin.extension.mcs.MCSException')
      str = [];
    else
      JE.printStackTrace();
      error('MCS:sgget:GetData','Error while getting the structure (see stack trace above):\n %s', ...
        char(JE.toString()))
    end
  end
  
catch ME
  if isa(ME,'matlab.exception.JavaException')
    ex = ME.ExceptionObject; % Java exception
    if isa(ex,'java.lang.IllegalStateException')
      error('MCS:sgget:GetData','StratiGraph returned the error:\n  %s',...
            char(ex.getMessage()))
    else
      rethrow(ME)
    end
  else
    rethrow(ME)
  end
end

varargout{1} = str;

if isempty(str)
  % If SG structure is empty clear sgcom structure
  sgc = sgcom();
  sgdata(winID,sgc);
  [varargout{2:nargout}] = deal([]);
  warning('MCS:sgget:StructureCleared',...
          'No active canonical structure in the StratiGraph window %s.\n The associated canonical structure in Matlab is cleared.',label)
  return
elseif ~strcmp(class(str),class(getstruct(sgc)))
  % If SG structure and sgcom structure are not the same type,
  % assign the SG structure to the sgcom structure
  sgc = sgcom(str,jstrCont.getStrata().getSupClass());
  sgdata(winID,sgc);
  [varargout{2:nargout}] = deal([]);
  warning('MCS:sgget:StructureChanged',...
          'The associated canonical structure in Matlab to the StratiGraph window %s has been changed to %s.',label,class(str))
  return
end



