function varargout = sggetconstraints(varargin)
%SGGETCONSTRAINTS Return a list of available setup constraints.
%   SGGETCONSTRAINTS list all setups and available constraints in
%   StratiGraph to the command window.
%
%   Constr = SGGETCONSTRAINTS(Struct) returns for a canonical structure
%   object the list Constr with all available constraints for the
%   corresponding setup in StratiGraph, where Struct can be a canonical
%   structure object or the name of the canonical structure class as a
%   string. The returned list is a cell-array of strings.
%
%   See also SGSET.

% This file is part of the Matlab plugin for StratiGraph.
% This file is freely-available under the modified
% BSD license.

narginchk(0,1);
nargoutchk(0,1);

if nargin > 0
  obj = varargin{1};

  if ~isa(obj,'mcsstruct') && ~ischar(obj)
    error('The first argument must be a string or a canonical structure object.');
  end
  
  if ischar(obj)
    sobj = meta.class.fromName(obj);
    pSetup = findobj(sobj.PropertyList,'Name','SGSetupName');
    setupName{1} = pSetup.DefaultValue; % get SG setup name
  elseif isa(obj,'mcsstruct')
    setupName{1} = obj.SGSetupName;
  end
  
  if isempty(setupName{1})
    warning('MCS:sggetconstraints:InvalidSetup','Setup is not implemented in StratiGraph.');
    varargout{1} = [];
    return
  end
else
  if nargout > 0
    error('With no input arguments the function do not return anything.');
  end
  
  curdir = fileparts(which('mcsstruct'));
  
  ds = dir([curdir '/*.m']);
  k = 0;
  setupName = {};
  for i = 1:length(ds)
    if endsWith(ds(i).name,{'struct.m'})
      [~,str] = fileparts(ds(i).name);
      sobj = meta.class.fromName(str);
      pSetup = findobj(sobj.PropertyList,'Name','SGSetupName');
      if ~pSetup.Abstract && ~isempty(pSetup.DefaultValue)
        k = k+1;
        setupName{k} = pSetup.DefaultValue; % get SG setup name
      end
    end
  end
end
  
try
  [~, winID, ~] = sgdata('');
catch e
  if strcmp(e.identifier,'MCS:sgdata:NoInstance')
    error('MCS:sggetconstraints:NoInstance',['StratiGraph must be ' ...
                        'started to get available constraints.']);
  end
  rethrow(e)
end

% Check that MCS Toolbox Integrator is loaded into StratiGraph
import se.umu.stratigraph.core.PluginManager;
if ~PluginManager.isLoaded(winID,'MCS Toolbox Integrator')
  error('MCS:sggetconstraints:ExtensionNotLoaded',...
        'StratiGraph extension ''MCS Toolbox Integrator'' is not loaded.')
end

import se.umu.stratigraph.Main;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestInfoContainer;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataType;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequest;
  
try 
  for k = 1:length(setupName)
    % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer
    jrc  = MCSRequestInfoContainer(MCSRequestDataType.GET_EXPANDERS, ...
      setupName{k});
    % Call se.umu.stratigraph.plugin.extension.mcs.MCSRequest
    jre  = MCSRequest(winID, jrc);
    
    jres = Main.callExtension(jre); % Get Result
    jstringCont = jres.getData(); % Get StringContainer
    if nargin > 0
      if ~jstringCont.hasData()
        constr = {};
      else
        constr = cellfun(@(x)strrep(x,'Expander',''),...
          cell(jstringCont.getStrings()),'UniformOutput',false);
      end
    else
      constr = cellfun(@(x)strrep(x,'Expander',''),...
        cell(jstringCont.getStrings()),'UniformOutput',false);
      fprintf('Setup ''%s'' with the constraints:\n', setupName{k});
      for c=1:length(constr)
        fprintf('   %s\n',constr{c});
      end
    end
    
  end
catch e
  if isa(e,'matlab.exception.JavaException')
    ex = e.ExceptionObject; % Java exception
    if isa(ex,'java.lang.IllegalStateException')
      error('MCS:sggetconstraints:GetData','StratiGraph returned the error:\n  %s', char(ex.getMessage()))
    else
      rethrow(e)
    end
  else
    rethrow(e)
  end
end

if nargout == 1
  varargout{1} = constr;
end

